package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.ApplicationTypeEnum;
import com.zanxiang.common.enums.DeleteEnum;
import com.zanxiang.common.enums.StatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.bo.PayApplicationBO;
import com.zanxiang.manage.domain.dto.PayApplicationDTO;
import com.zanxiang.manage.domain.params.PayApplicationAddUpdateParam;
import com.zanxiang.manage.domain.params.PayApplicationListNoPageParam;
import com.zanxiang.manage.domain.params.PayApplicationListParam;
import com.zanxiang.manage.domain.vo.PayApplicationChoiceVO;
import com.zanxiang.manage.domain.vo.PayApplicationVO;
import com.zanxiang.manage.service.PayApplicationService;
import com.zanxiang.manage.service.api.MiniAppletApiService;
import com.zanxiang.mybatis.entity.PayApplication;
import com.zanxiang.mybatis.mapper.PayApplicationMapper;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 支付应用管理
 *
 * @author xufeng
 * @date 2022-06-20 15:16
 */
@Service
public class PayApplicationServiceImpl extends ServiceImpl<PayApplicationMapper, PayApplication> implements PayApplicationService {

    @Autowired
    private MiniAppletApiService miniAppletApiService;

    /**
     * 支付应用选择列表
     *
     * @return : 返回所有支付应用信息
     */
    @Override
    public List<PayApplicationChoiceVO> payApplicationChoiceList() {
        List<PayApplication> payApplicationList = super.list(new LambdaQueryWrapper<PayApplication>()
                .select(PayApplication::getId, PayApplication::getName));
        return BeanUtils.copyList(payApplicationList, PayApplicationChoiceVO.class);
    }

    /**
     * 分页获取应用列表
     *
     * @param param : 查询参数
     * @return : 返回列表
     */
    @Override
    public IPage<PayApplicationVO> list(PayApplicationListParam param) {
        return page(param.toPage(), new LambdaQueryWrapper<PayApplication>()
                .like(Strings.isNotBlank(param.getName()), PayApplication::getName, param.getName())
                .eq(param.getType() != null, PayApplication::getType, param.getType())
                .like(Strings.isNotBlank(param.getAppId()), PayApplication::getAppId, param.getAppId())
                .like(Strings.isNotBlank(param.getCompany()), PayApplication::getCompany, param.getCompany())
                .orderByDesc(PayApplication::getCreateTime))
                .convert(this::toVo);
    }

    /**
     * 给签证官
     *
     * @param payApplication 支付应用程序
     * @return {@link PayApplicationVO}
     */
    private PayApplicationVO toVo(PayApplication payApplication) {
        PayApplicationVO applicationVO = BeanUtils.copy(payApplication, PayApplicationVO.class);
        if (Objects.isNull(applicationVO)) {
            return null;
        }
        applicationVO.setTypeName(ApplicationTypeEnum.getDescByType(applicationVO.getType()));
        return applicationVO;
    }

    /**
     * 支付应用添加或者更新
     *
     * @param param : 参数
     * @return : 返回结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean addOrUpdate(PayApplicationAddUpdateParam param) {
        Long id = param.getId();
        //更新
        if (id != null) {
            PayApplication payApplication = super.getById(id);
            if (payApplication == null) {
                throw new BaseException("参数错误, 支付应用信息不存在");
            }
            //更新时, 应用id不可重复
            if (!Objects.equals(payApplication.getAppId(), param.getAppId())) {
                //判断应用是否已经存在
                int count = super.count(new LambdaQueryWrapper<PayApplication>()
                        .eq(PayApplication::getAppId, param.getAppId()));
                if (count > 0) {
                    throw new BaseException("应用id已经存在");
                }
            }
            payApplication.setName(param.getName());
            payApplication.setType(param.getType());
            payApplication.setAppSecret(param.getAppSecret());
            payApplication.setVersion(param.getVersion());
            payApplication.setAppDesc(param.getAppDesc());
            payApplication.setAppUrl(param.getAppUrl());
            payApplication.setAppLogo(param.getAppLogo());
            payApplication.setMiniCode(param.getMiniCode());
            payApplication.setAppId(param.getAppId());
            payApplication.setPrimevalId(param.getPrimevalId());
            payApplication.setCompany(param.getCompany());
            payApplication.setLoginName(param.getLoginName());
            payApplication.setLoginPassword(param.getLoginPassword());
            payApplication.setManagePerson(param.getManagePerson());
            payApplication.setRemark(param.getRemark());
            payApplication.setIsDelete(DeleteEnum.NO.getCode());
            payApplication.setStatus(StatusEnum.YES.getCode());
            payApplication.setUpdateTime(LocalDateTime.now());
            return super.updateById(payApplication);
        }
        //判断名称是否已经存在
        int count = super.count(new LambdaQueryWrapper<PayApplication>()
                .eq(PayApplication::getAppId, param.getAppId()));
        if (count > 0) {
            throw new BaseException("应用id已经存在");
        }
        //创建
        return super.save(PayApplication.builder()
                .name(param.getName())
                .type(param.getType())
                .appId(param.getAppId())
                .appSecret(param.getAppSecret())
                .primevalId(param.getPrimevalId())
                .company(param.getCompany())
                .loginName(param.getLoginName())
                .loginPassword(param.getLoginPassword())
                .managePerson(param.getManagePerson())
                .remark(param.getRemark())
                .status(StatusEnum.YES.getCode())
                .isDelete(DeleteEnum.NO.getCode())
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build());
    }

    /**
     * 获取游戏应用二维码图片地址
     *
     * @param id : 应用id
     * @return : 返回游戏二维码
     */
    @Override
    public String getQrCode(Long id) {
        PayApplication payApplication = super.getById(id);
        if (payApplication == null) {
            throw new BaseException("支付应用信息不存在");
        }
        return miniAppletApiService.getQrCode(payApplication.getAppId(), payApplication.getAppSecret(), payApplication.getName());
    }

    /**
     * 根据主键更新状态
     *
     * @param id     : 主键id
     * @param status : 状态
     * @return : 返回更新结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean statusUpdate(Long id, Integer status) {
        return super.update(new LambdaUpdateWrapper<PayApplication>()
                .set(PayApplication::getStatus, status)
                .set(PayApplication::getUpdateTime, LocalDateTime.now())
                .eq(PayApplication::getId, id));
    }

    /**
     * 删除
     *
     * @param id : 要删除的信息id
     * @return : 返回删除结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteById(Long id) {
        return super.removeById(id);
    }


    /**
     * 支付应用情况
     *
     * @param payApplicationId   : 支付应用程序id
     * @param payApplicationType : 支付应用程序类型
     * @param appId              : 应用程序id
     * @param company            : 公司
     * @return {@link Map}<{@link Long}, {@link PayApplicationDTO}>
     */
    @Override
    public Map<Long, PayApplicationDTO> payApplicationCondition(Integer payApplicationId, Integer payApplicationType,
                                                                String appId, String company) {
        //条件判断
        if (payApplicationId == null && payApplicationType == null && Strings.isBlank(appId) && Strings.isBlank(company)) {
            return null;
        }
        List<PayApplication> payApplicationList = this.list(new LambdaQueryWrapper<PayApplication>()
                .eq(payApplicationId != null, PayApplication::getId, payApplicationId)
                .eq(payApplicationType != null, PayApplication::getType, payApplicationType)
                .like(Strings.isNotBlank(appId), PayApplication::getAppId, appId)
                .like(Strings.isNotBlank(company), PayApplication::getCompany, company));
        if (CollectionUtils.isEmpty(payApplicationList)) {
            return Collections.emptyMap();
        }
        return BeanUtils.copyList(payApplicationList, PayApplicationDTO.class)
                .stream().collect(Collectors.toMap(PayApplicationDTO::getId, Function.identity()));
    }

    /**
     * 支付应用程序地图
     *
     * @return {@link Map}<{@link Long}, {@link PayApplicationDTO}>
     */
    @Override
    public Map<Long, PayApplicationDTO> payApplicationMap() {
        List<PayApplication> payApplicationList = this.list();
        if (CollectionUtils.isEmpty(payApplicationList)) {
            return Collections.emptyMap();
        }
        return BeanUtils.copyList(payApplicationList, PayApplicationDTO.class)
                .stream().collect(Collectors.toMap(PayApplicationDTO::getId, Function.identity()));
    }


    //------------------------------------------------- 准备删除的旧代码 -------------------------------------------------

    @Override
    public Boolean saveOrUpdate(PayApplicationBO bo) {
        PayApplication app = BeanUtils.copy(bo, PayApplication.class);
        if (bo.getIsDelete() != null && bo.getIsDelete() == DeleteEnum.YES.getCode()) {
            return removeById(bo.getId());
        }
        return saveOrUpdate(app);
    }

    @Override
    public List<PayApplicationVO> listNoPage(PayApplicationListNoPageParam bo) {
        PayApplication app = BeanUtils.copy(bo, PayApplication.class);
        app.setStatus(StatusEnum.YES.getCode());
        List<PayApplication> list = list(new LambdaQueryWrapper<PayApplication>().setEntity(app).orderByDesc(PayApplication::getCreateTime));
        if (list.size() == 0) {
            return null;
        }
        return BeanUtils.copyList(list, PayApplicationVO.class);
    }
}
